import 'package:cryptography/cryptography.dart';
import 'package:equatable/equatable.dart';
import 'package:meta/meta.dart';
import 'package:ticket/ticket.dart';

/// Utility class for generating many [Stub]s using the same [KeyPair] and other options
///
///
/// [hashAlgorithm] defaults to [blake2b]
///
/// Instantiate a [SGen] instance, and call [generate()] on that to generate [Stub]s while only needing to provide an error field, payload, and transaction identifier, the rest is filled in by [SGen]
class SGen extends Equatable {
  SGen._constructor(this.keyPair, this.hashAlgorithm);

  factory SGen(
      {@required KeyPair keyPair, HashAlgorithm hashAlgorithm = blake2b}) {
    return SGen._constructor(keyPair, hashAlgorithm);
  }

  final KeyPair keyPair;
  final HashAlgorithm hashAlgorithm;

  /// Generate a [Stub] using the provided [transactionIdentifier], [error], and [payload].
  ///
  /// [payload] defaults to [null] and [error] defaults to `false`.
  Stub generate(
      {dynamic payload,
      bool error,
      @required List<int> transactionIdentifier}) {
    return Stub.generate(
        transactionIdentifier: transactionIdentifier,
        keyPair: keyPair,
        error: error,
        hashAlgorithm: hashAlgorithm);
  }

  @override
  List<Object> get props => [keyPair, hashAlgorithm];
}
