import 'dart:convert';

import 'package:equatable/equatable.dart';

/// Represents the infoBody (information body) of a [Stub].
class StubInfoBody extends Equatable {
  StubInfoBody(this.transactionIdentifier, this.payload, this.error);

  /// The identifier of the transaction the [Stub] is involved in
  final List<int> transactionIdentifier;

  /// Response payload - can be anything serializable
  final dynamic payload;

  /// Whether there was an error when fulfilling the [Ticket] or not
  final bool error;

  /// Checks if the [InfoBody] is valid or not
  bool isValid() {
    // check if transactionIdentifier is at least 6 bytes
    if (transactionIdentifier.length < 6) return false;

    // check if error is null
    if (error == null) return false;

    return true;
  }

  /// Returns a [Map] representing the [StubInfoBody]
  Map toMap() {
    return {
      'transactionIdentifier': transactionIdentifier,
      'payload': payload,
      'error': error
    };
  }

  /// Returns a JSON string representing the [StubInfoBody]
  String toJson() {
    return jsonEncode(toMap());
  }

  /// Returns a [StubInfoBody] instance from a [Map] generated by [toMap()]
  factory StubInfoBody.fromMap(Map map) {
    return StubInfoBody(List<int>.from(map['transactionIdentifier']),
        map['payload'], map['error']);
  }

  /// Returns a [StubInfoBody] instance from a JSON string
  factory StubInfoBody.fromJson(String json) {
    return StubInfoBody.fromMap(jsonDecode(json));
  }

  @override
  List<Object> get props => [transactionIdentifier, payload, error];
}
